import jas.engine.*;
import jas.space.*;
import jas.events.ISimEventListener;

import java.util.*;
import java.awt.Point;

/**
 * The agent for the Heat Bugs simulation. This pretty much follows the
 * Swarm code.
 *
 * @author Swarm Project, JAS implementation: Michele Sonnessa
 * @see HeatBugsModel
 */

public class HeatBug extends DigitalTurtle implements ISimEventListener
{
  public static final int EVENT_STEP = 0;

  private double unhappiness = 0;
  private int idealTemp, outputHeat;
  private float randomMoveProb;

  private HeatSpace space;

  public HeatBug(int id, HeatSpace space, IObjGrid world, int x,
    int y, int idealTemp, int outputHeat, float randomMoveProb, SimModel owner)
  {
	super(id, world, x, y);
    this.idealTemp = idealTemp;
    this.outputHeat = outputHeat;
    this.randomMoveProb = randomMoveProb;
    this.space = space;
  }

  private void moveToExtreme(long heatHere)
  {
	TreeMap temps = new TreeMap();

	setRandomHeading();

	for (int i = 0; i < 8; i++)
	{
		turnCardinalRight(1);
		int nX = getNextX();
		int nY = getNextY();

		if (grid.countObjectsAt(nX, nY) == 0)
			temps.put(new Long((long) space.getDbl(nX, nY)), new Integer(getHeading()));
	}

	int type = (heatHere < idealTemp) ? HeatSpace.HOT : HeatSpace.COLD;
	Integer newHead;
	if (type == HeatSpace.HOT)
		newHead = (Integer) temps.get(temps.lastKey());
	else
		newHead = (Integer) temps.get(temps.firstKey());

	setHeading(newHead.intValue());
	forward();
  }

  private void moveRandom()
  {
	setRandomHeading();

	for (int i = 0; i < 8; i++)
	{
		turnCardinalRight(1);
		if (leap())
			return;
	}
  }

  public void step()
  {

    long heatHere = (long) space.getDbl(x, y);

	unhappiness = (double) Math.abs(idealTemp - heatHere) / space.MAX;

    space.addHeat(x, y, outputHeat);
    if (unhappiness == 0)
		return;

    if (Sim.getRnd().getDblFromTo(0.0, 1.0) < randomMoveProb)
    	moveRandom();
    else
    	moveToExtreme(heatHere);

  }

  public void setIdealTemp(int idealTemp)
  {
	  this.idealTemp = idealTemp;
  }

  public void performAction(int actionType)
  {
    switch (actionType)
    {
      case EVENT_STEP:
        step();
        break;
      default:
        System.err.println("No action for HeatBug");
    }
  }

}
