import jas.engine.*;
import jas.events.*;
import jas.plot.*;
import jas.space.*;

import jas.net.*;
import jas.io.SVGGenerator;

import java.awt.Color;

public class HeatBugsSrvObs extends SimModel
{
  HeatBugsSrvModel model;

  private LayeredSurfaceFrame display;
  private TimePlot plotter;

  private InformationServer server;
  private int port;

  public HeatBugsSrvObs(int defPort)
  {
	port = defPort;
  }

  public HeatBugsSrvObs()
  {
	this(9990);
  }

  public void setParameters()
  {
  }

  public void buildModel()
  {
    model = (HeatBugsSrvModel) Sim.engine.getModelWithID("HeatBugsSrvModel");
    if (model == null)
      System.exit(1);
    buildDisplay();
    buildServer();
    buildActions();
  }

  private void buildServer()
  {
	server = new InformationServer(port);
	System.out.println("Information server installed on port" + port);
  }

  public void checkRegenerate()
  {
	  if (server.receiveInfo("Anonymous User", "id06") != null &&
	  	  !server.receiveInfo("Anonymous User", "id06").equals(""))
	  {
		model.minIdealTemp = Integer.parseInt(server.receiveInfo("Anonymous User", "id06"));
		model.maxIdealTemp = Integer.parseInt(server.receiveInfo("Anonymous User", "id07"));
	  	model.regeneratePreferences();
	  	server.jReceive("Anonymous User", "id06", "");
  	  }

  }

  public void updateServer()
  {
	server.addInfo("Anonymous User", "id01", SVGGenerator.generateAsPicture(plotter.getPlot()));
	server.addInfo("Anonymous User", "id02", SVGGenerator.generateAsPicture(display.getSpacePanel()));
	server.addInfo("Anonymous User", "id03", Sim.getTime().toString());
	server.addInfo("Anonymous User", "id04", "" + (int)(model.steps - Sim.getAbsoluteTime()));
  }

  private void buildDisplay() {

    ColorMap cRange = new ColorRangeMap(64, 0, model.space.MAX, Color.red);
    //ColorMap cRange = new ColorDualRangeMap(500, ColorMap.COLOR_BLUE, ColorMap.COLOR_RED,
    //                0, space.MAX / 2, space.MAX );
    //ColorMap cRange = new ColorTripleRangeMap(500, ColorMap.COLOR_BLUE, ColorMap.COLOR_GREEN,
    //                    ColorMap.COLOR_RED, 0, space.MAX / 2, space.MAX );

    LayerDblGridDrawer d = new LayerDblGridDrawer("Heat", model.space, cRange);
    LayerObjGridDrawer s = new LayerObjGridDrawer("Bugs", model.world,
                                              java.awt.Color.yellow);

    display = new LayeredSurfaceFrame(model.worldXSize, model.worldYSize, 4);
    display.addLayer(d);
    display.addLayer(s);
    addSimWindow(display);
	//display.setVisible(false);

    plotter = new TimePlot("Unhappiness", model.unhapTime);
    plotter.setDataWindow(500);
    addSimWindow(plotter);
    //plotter.setVisible(false);
  }

  private void buildActions()
  {
    SimGroupEvent s;

    s = eventList.scheduleGroup(0, 1);
    s.addEvent(display, Sim.EVENT_UPDATE);
    s.addEvent(plotter, Sim.EVENT_UPDATE);
	s.addEvent(this, "checkRegenerate");

    eventList.scheduleSimple(0, 15, this, "updateServer");
  }

  public static void main(String[] args)
  {
	int defPort = 9990;
	int defTimeTreshold = 150;
	int steps = 5000;

	if (args.length > 0)
	{
		defPort = Integer.parseInt(args[0]);
		defTimeTreshold = Integer.parseInt(args[1]);
		steps = Integer.parseInt(args[2]);
	}
	else
	{
		System.out.println("args: [tcpPort timeTreshold steps]");
		System.out.println("Default values: 9990 150 5000");
	}

  	  HeatBugsSrvModel m = new HeatBugsSrvModel();
      HeatBugsSrvObs p = new HeatBugsSrvObs(defPort);
		m.steps = steps;

      SimEngine eng = new SimEngine();

	  eng.getEventList().setEventTimeTreshold(defTimeTreshold);
  	  eng.addModel(m);
      eng.addModel(p);
      eng.startGUI(false);

      eng.buildModels();
      eng.start();
  }
}

